module AuthActivity
  module AuthLogger
    extend ActiveSupport::Concern

    included do
      after_action :log_login_attempt
      after_action :log_password_request
      around_action :log_logout_action

      before_action :set_current
    end

    private

    def set_current
      AuthActivity::Current.request_id = request.uuid
      AuthActivity::Current.user_agent = request.user_agent
      AuthActivity::Current.ip_address = request.ip
    end

    def user
      return current_user if signed_in?
      return if email.blank?

      ::User.find_by(email: email)
    end

    def email
      params.dig(:session, :email) || params.dig(:password, :email) || current_user&.email
    end

    def login_attempt?
      controller_name == "sessions" && action_name == "create"
    end

    def password_request_attempt?
      controller_name == "passwords" && action_name == "create"
    end

    def logout_action?
      controller_name == "sessions" && action_name == "destroy"
    end

    def log_login_attempt
      return unless login_attempt?

      if signed_in?
        AuthEvent.succesful_login.create(trackable: user, metadata: { email: email })
      else
        AuthEvent.failed_login.create(trackable: user, metadata: { email: email })
      end
    end

    def log_password_request
      return unless password_request_attempt?

      if user.present?
        AuthEvent.succesful_password_request.create(trackable: user, metadata: { email: email })
      else
        AuthEvent.failed_password_request.create(metadata: { email: email })
      end
    end

    def log_logout_action
      if user.present? && logout_action?
        user = current_user
        yield
        AuthEvent.logout.create(trackable: user, metadata: { email: user.email })
      else
        yield
      end
    end
  end
end
