module AuthActivity::User
  AUTH_ATTRIBUTES = %w[email encrypted_password].freeze
  FILTER_ATTRIBUTES = %w[encrypted_password].freeze
  extend ActiveSupport::Concern

  included do
    after_update :log_changed, :send_password_changed_email
    after_create :log_created
    after_destroy :log_destroyed
  end

  def plausible_name
    try(:full_name) || try(:name) || [try(:first_name), try(:last_name)].compact.join(" ")
  end

  private

  def log_created
    AuthActivity::AuthEvent.user_created.create!(trackable: self)
  end

  def log_changed
    return unless changed_auth_attributes.any?

    AuthActivity::AuthEvent.
      attributes_changed.create!(trackable: self,
                                 metadata: { attributes_changed: filtered_auth_changes })
  end

  def log_destroyed
    AuthActivity::AuthEvent.user_destroyed.create!(trackable: self)
  end

  def changed_auth_attributes
    saved_changes.keys & AUTH_ATTRIBUTES
  end

  def filtered_auth_changes
    auth_changes.update(auth_changes) do |k, v|
      return { k => ["[filtered]", "[filtered]"] } if k.in? FILTER_ATTRIBUTES

      { k => v }
    end
  end

  def auth_changes
    saved_changes.select { |k| k.in?(changed_auth_attributes) }
  end

  def send_password_changed_email
    return unless AuthActivity::Engine.config.enable_password_changed_email
    return unless saved_changes.key?(:encrypted_password)

    UserMailer.password_changed(self).deliver_later
  end
end
